<?php

require_once __DIR__ . "/../bootstrap/app.php";
require_once __DIR__ . '/../func/Log.php';
require_once __DIR__ . "/../func/Zonewatch.php";

use Stomp\Client;
use Stomp\Exception\StompException;

## check for updated zones
$zones = Zonewatch::getLastZones(2);
foreach ($zones as $domain) {
    $check = $db->table($table)->where('domain_name', $domain)->count('dt_id', 'total')->get()->total;
    if ($check != 0) {
        $punydomain = convertIDN($domain);
        $logger->info("DNS records of $punydomain was updated, queued domain for transfer.");
        $db->table($table)->where(array(
            'domain_name' => $domain,
            'is_sended' => 1,
        ))->update(array(
            'is_sended' => 0,
            'domain_action' => 1,
        ));
    }
}

if (is_dir("/var/named/")) {
    $domain_path = "/var/named/";
} else if (is_dir("/etc/bind/")) {
    $domain_path = "/etc/bind/";
} else {
    $logger->error("DNS Zone files directory could not be found!");
    exit;
}

# Get accidentally deleted domain entries
$deleted_domains = $db->table($table)->select('*')->where(array('is_deleted' => 1))->limit(50)->getAll();
$deleted_domain_count = count($deleted_domains);
if ($deleted_domain_count > 0) {
    foreach ($deleted_domains as $domain) {
        $zoneFile = $domain_path . $domain->domain_name . ".db";
        if ($domain->domain_action != 2 && is_file($zoneFile)) {
            $updateData = array('is_sended' => 0, 'is_deleted' => 0);
            $db->table($table)->where('domain_name', $domain->domain_name)->update($updateData);
            $punydomain = convertIDN($domain->domain_name);
            $logger->info("The accidentally deleted domain ($punydomain) has been marked to resent!");
        }
    }
}

# Get domains for transfer
$domains = $db->table($table)->select('*')->where(array('is_deleted' => 0, 'is_sended' => 0))->limit(50)->getAll();
$domain_count = count($domains);
$settings = $db->table('settings')->select('*')->where('is_deleted', 0)->get();

if ($domain_count == 0) {
    exit;
}

$logger->addInfo("Transfer process started");

if (empty($settings->controller)) {
    $logger->addInfo("Controller was not added in settings, transfer process terminated!");
    exit();
}

$controllerip = $settings->controller;
$failover = $settings->failover;

$activeServerIp = trim(getActiveServer($controllerip, $failover));

if (empty($activeServerIp)) {
    $logger->error("No active controller or failover servers found");
    exit;
}

$activePort = "61613";
$delay = 5;
$public_ip = trim(shell_exec("curl -s http://rep0.admin-ahead.com/sources/verify/getip.php 2>/dev/null"));
$private_ip = trim(shell_exec("$appPath/bin/ctl gdnsbin getlocalip"));

# connecting to AMQ
$queue = '/queue/gdns_domain_queue';

$stomp = new Client("tcp://$activeServerIp:$activePort");

try {
    $stomp->connect();
} catch (StompException $e) {
    $logger->error("Connection to $activeServerIp was failed!");
    echo $e->getMessage() . "\n";
    exit;
}

if ($domain_count == 0) {
    exit;
}

$logger->addInfo($domain_count . ' domain(s) selected for transfer');

$dahost = shell_exec("hostname -f 2>/dev/null");
if (!$dahost) {
    $dahost = gethostname();
}
$dahost = strtolower($dahost);

foreach ($domains as $domain) {
    $punydomain = convertIDN($domain->domain_name);
    $zoneFile = $domain_path . $domain->domain_name . ".db";
    if (is_file($zoneFile) || (!is_file($zoneFile) && $domain->domain_action == 2)) {

        $file_content = $domain->domain_action == 2 ? 'zone-remove' : trim(shell_exec("$appPath/bin/ctl gdnsbin readzone $domain->domain_name"));
        $sendData = array("zone_file" => $file_content, "domain_name" => strtolower($domain->domain_name), "public_ip" => $public_ip, "domain_action" => $domain->domain_action, "request_type" => $domain->request_type, "private_ip" => $private_ip, 'host_name' => $dahost);
        sleep(1);
        $stomp->send($queue, serialize($sendData), array('persistent' => 'true', 'priority' => $domain->priority, 'domain' => $domain->domain_name));

        $updateData = array('is_sended' => 1, 'priority' => 4, 'timestamp' => getTimeNow());
        if ($domain->domain_action == 2) {
            $updateData['is_deleted'] = 1;
        }
        $db->table($table)->where('domain_name', $domain->domain_name)->update($updateData);

        if ($domain->domain_action == 2) {
            $logger->info('Domain ' . $punydomain . ' has been marked for deletion');
        } else {
            $logger->info('Successfully read zone file of ' . $punydomain);
        }

    } else {
        if ($domain->domain_action != 2) {
            if ($db->table($table)->where('domain_name', $domain->domain_name)->update(array(
                "timestamp" => getTimeNow(),
                "is_sended" => 0,
                "domain_action" => 2)
            )) {
                $logger->error('Error reading zone file of the domain ' . $punydomain . ' moved domain to delete queue!');
            } else {
                $logger->error('Error reading zone file of the domain ' . $punydomain);
            }
        }
    }
}

# close connection
$stomp->disconnect();
$logger->addInfo("controller connection terminated");
