#!/bin/bash

#############################################################################
# Copyright(c) 2022 Admin-Ahead Server Technologies. All rights Reserved.   #
# This code is subject to the Admin-Ahead license.                          #
# Unauthorized copying is prohibited.                                       #
# Support: support@admin-ahead.com                                          #
# Web: https://admin-ahead.com                                              #
#############################################################################

export PLGPATH='/usr/local/directadmin/plugins'
export APP='gdns-transferserver'
export PLGDIR=$PLGPATH/$APP
export SYSHOOK=/usr/local/directadmin/scripts/custom
USER_MYCNF=/usr/local/directadmin/conf/my.cnf
DEFAULT_MYCNF=/root/.my.cnf

[[ ! -d "/usr/local/directadmin/" ]] && echo 'Invalid Server!' && exit 0

checkHook() {
	local hook=$1
	hookfile="${SYSHOOK}/$hook"
	if [[ -s "$hookfile" ]]; then
		shbg=$(sed -n '1p' $hookfile)
		if [[ "${shbg:0:3}" == "#!/" ]]; then
			return 2
		else
			return 1
		fi
	fi
	return 0
}

removeOldHook() {
    hooks=("domain_create_post" "domain_destroy_pre" "domain_change_post" "subdomain_create_post" "subdomain_destroy_pre" "user_create_post" "dns_write_post" "domain_pointer_create_pre" "domain_pointer_destroy_pre" "domain_modify_post" "dns_create_post" "dns_delete_post")
    for hook in  ${hooks[@]}; do
        if [[ -f "$SYSHOOK/${hook}.sh" ]]; then
            if [[ $(grep -wc "#gdns-script-check" $SYSHOOK/${hook}.sh) -ne 0 ]]; then
                rm -f $SYSHOOK/${hook}.sh 
            else
                checkHook ${hook}.sh > /dev/null 2>&1
                if [[ $? -eq 0 ]]; then
                    sed -i '/gdns\//d' $SYSHOOK/${hook}.sh > /dev/null 2>&1
                fi
            fi
        fi
    done
}

updateHookPath() {
    hooks=("domain_create_post" "domain_destroy_pre" "domain_change_post" "subdomain_create_post" "subdomain_destroy_pre" "user_create_post" "dns_write_post" "domain_pointer_create_pre" "domain_pointer_destroy_pre" "domain_modify_post" "dns_create_post" "dns_delete_post")
    for hook in  ${hooks[@]}; do
        if [[ -f "$SYSHOOK/${hook}.sh" ]]; then
            if [[ $(grep -wc "#gdns-script-check" $SYSHOOK/${hook}.sh) -ne 0 ]]; then
                sed -i 's~gdns-transferserver/scripts/hooks~gdns-transferserver/hooks~g' $SYSHOOK/${hook}.sh > /dev/null 2>&1
            else
                checkHook ${hook}.sh > /dev/null 2>&1
                if [[ $? -eq 0 ]]; then
                    sed -i 's~gdns-transferserver/scripts/hooks~gdns-transferserver/hooks~g' $SYSHOOK/${hook}.sh > /dev/null 2>&1
                fi
            fi
        fi
    done  
}

chown -R diradmin.diradmin $PLGDIR
cd $PLGDIR
find -type d -print0 | xargs -0 chmod 755 >/dev/null 2>&1
find -type f -print0 | xargs -0 chmod 644 >/dev/null 2>&1
chmod -R +x $PLGDIR/hooks/* $PLGDIR/app/bin/* $PLGDIR/admin/index.raw >/dev/null 2>&1
chmod 777 plugin.conf >/dev/null 2>&1
if [[ $(/usr/local/directadmin/directadmin v | awk '{print $3}' | cut -b 3-6 | tr -d .) -ge 160 ]]; then
    updateHookPath
else
    removeOldHook
fi

mysql --defaults-file=$USER_MYCNF -e ";" >/dev/null 2>&1
response_conf=$?
if [[ $response_conf -eq 0 ]];then
    export mysql_param='--defaults-file'
else
    export mysql_param='--defaults-extra-file'
fi
mysql $mysql_param=$USER_MYCNF -e ";" >/dev/null 2>&1
response_conf=$?
mysql -e ";"
response_default=$?
default_conf=0
if [[ -s $USER_MYCNF ]] && [[ $response_conf -eq 0 ]];then
	export MYCNF=$USER_MYCNF
elif [[ -s $DEFAULT_MYCNF ]] && [[ $response_default -eq 0 ]];then
	export MYCNF=$DEFAULT_MYCNF
	default_conf=1
else
	outInfo "Failed to get MySQL Details!"
	exit 100
fi

[[ $response_conf -eq 0 ]] && mysql $mysql_param=$MYCNF -e "ALTER TABLE gdns_transferserver.domain_transfer ADD COLUMN priority int(11) DEFAULT 4 COMMENT 'ActiveMQ priority';" >/dev/null 2>&1
[[ $response_default -eq 0 ]] && mysql -e "ALTER TABLE gdns_transferserver.domain_transfer ADD COLUMN priority int(11) DEFAULT 4 COMMENT 'ActiveMQ priority';" >/dev/null 2>&1
echo -e '\n <b>Successfully updated gDNS trasnfer server plugin.</b>'